// 界面管理

var UIManager = function () {
    this.uiStack = [];// 已经打开的界面栈
    this.panelCache = {};// 缓存的面板
    this.classmap = {};

    this.panelRoot = cc.find("Canvas/PanelRoot");// 根节点

    this.swallowLayer = cc.find("Canvas/touchSwallowLayer");
    this.swallowNum = 0;

    this.registeUI();

    G.eventCenter.on(g_eventTypes.UIEvent.showUI, this.showUI, this);
    G.eventCenter.on(g_eventTypes.UIEvent.hideUI, this.hideUI, this);
    G.eventCenter.on(g_eventTypes.UIEvent.closeAllUI, this.closeAllUI, this);
    G.eventCenter.on(g_eventTypes.UIEvent.swallowTouch, this.swallowTouch, this);
    G.eventCenter.on(g_eventTypes.UIEvent.cancelSwallow, this.cancelSwallow, this);
};

UIManager.prototype = {
    constructor: UIManager,

    registeUI () {
        this.classmap[g_resource.mainPanel] = require("MainPanel");
    },

    add (panel) {
        var panelNode = panel.getRootNode();
        panelNode.parent = this.panelRoot;
        panelNode.setSiblingIndex(this.panelRoot.childrenCount - 1);
        if (this.uiStack.length > 0) {
            var oldTop = this.uiStack[this.uiStack.length - 1];
            oldTop.onCover();
        }
        this.uiStack.push(panel);
        G.eventCenter.dispatchEvent(g_eventTypes.UIEvent.UIOpened, panel.getResourceUrl());
    },

    // 关闭面板
    remove (panel) {
        panel.getRootNode().removeFromParent(false);// 不传参数false触摸会有问题哟
        var index = this.uiStack.indexOf(panel);
        if (-1 != index) {
            this.uiStack.splice(index, 1);
            if (this.uiStack.length > 0 && index == this.uiStack.length) {
            var top = this.uiStack[this.uiStack.length - 1];
            top.reShow();
            }
        }
        var url = panel.getResourceUrl();
        this.panelCache[url] = panel;
        G.eventCenter.dispatchEvent(g_eventTypes.coreEventType.UIClosed, url);
    },

    // 获取缓存的界面
    getUICache(url)
    {
        return this.panelCache[url]
    },

    showUI(url, ...args) {
        let panel = this.getUICache(url)
        if (panel) {
            panel.show(...args)
        } else {
            let asset = G.resourceLoader.getAsset(url)
            if (asset) {
                this.createUI(url, ...args)
            } else {
                let data = {}
                data.url = url
                data.callback = this.onLoadPanelComplete
                data.target = this
                data.data = args
                data.loadPanelType = LoadPanelType.windowPanel
                G.eventCenter.dispatchEvent(g_eventTypes.loadEvent.loadResource, data);
            }
        }
    },

    //关闭界面
    hideUI (url) {
        for (var i = 0; i < this.uiStack.length; i++) {
            var panelInfo = this.uiStack[i];
            var curUrl = panelInfo.getResourceUrl();
            if (url == curUrl) {
            panelInfo.hide();
            break;
            }
        }
    },

    onLoadPanelComplete (error, data) {
        if (error) {
            G.eventCenter.dispatchEvent(g_eventTypes.UIEvent.showTips, g_string.resourceLoadFailed);
            return;
        }
        this.createUI(data.url, ...data.data)
    },

    createUI (url) {
        var asset = G.resourceLoader.getAsset(url);
        var rootNode = cc.instantiate(asset);
        if (!rootNode) {
            G.eventCenter.dispatchEvent(g_eventTypes.UIEvent.showTips, g_string.resourceLoadFailed);
            return;
        }
        rootNode.parent = this.panelRoot;
        var panelclass = this.classmap[url];
        var uiPanel = new panelclass(rootNode, url);
        uiPanel.show(...args)
    },

    closeAllUI () {
        for (var i = this.uiStack.length - 1; i >= 0; --i) {
            var panel = this.uiStack[i];
            panel.hide();
        }
    },

    getTopUI () {
        if (this.uiStack.length <= 0) return;
        return this.uiStack[this.uiStack.length - 1];
    },

    swallowTouch () {
        this.swallowNum++;
        this.swallowLayer.active = true;
    },

    cancelSwallow () {
        --this.swallowNum;
        if (this.swallowNum <= 0) {
            this.swallowNum = 0;
            this.swallowLayer.active = false;
        }
    }
};
module.exports = UIManager;