
cc.Class({
    extends: cc.Component,
    properties: {
      itemTemplate: {
        default: null,
        type: cc.Prefab
      },
      scrollView: {
        default: null,
        type: cc.ScrollView
      },
      componentName: "item",
      spacing: 0
    },
    onLoad: function onLoad() {
      this.content = this.scrollView.content;
      this.items = [];
      this.updateTimer = 0;
      this.updateInterval = .01;
      this.lastContentPosY = 0;
    },
    initialize: function initialize(totalCount) {
      this.totalCount = totalCount;
      this.spawnCount = Math.ceil(this.scrollView.node.height / (this.itemTemplate.data.height + this.spacing)) + 1;
      this.bufferZone = .5 * (this.itemTemplate.data.height + this.scrollView.node.height);
      this.content.height = this.totalCount * (this.itemTemplate.data.height + this.spacing) + this.spacing;
      for (var i = 0; i < this.spawnCount; ++i) {
        var item = cc.instantiate(this.itemTemplate);
        this.content.addChild(item);
        item.setPosition(0, -item.height * (.5 + i) - this.spacing * (i + 1));
        item.getComponent(this.componentName).updateItem(i);
        this.items.push(item);
      }
    },
    refresh: function refresh() {
      for (var i = 0; i < this.items.length; ++i) {
        var item = this.items[i].getComponent(this.componentName);
        var index = item.itemID;
        item.itemID = null;
        item.updateItem(index);
      }
    },
    getPositionInView: function getPositionInView(item) {
      var worldPos = item.parent.convertToWorldSpaceAR(item.position);
      var viewPos = this.scrollView.node.convertToNodeSpaceAR(worldPos);
      return viewPos;
    },
    update: function update(dt) {
      this.updateTimer += dt;
      if (this.updateTimer < this.updateInterval) return;
      this.updateTimer = 0;
      var items = this.items;
      var buffer = this.bufferZone;
      var isDown = this.scrollView.content.y < this.lastContentPosY;
      var offset = (this.itemTemplate.data.height + this.spacing) * items.length;
      for (var i = 0; i < items.length; ++i) {
        var viewPos = this.getPositionInView(items[i]);
        if (isDown) {
          if (viewPos.y < -buffer) {
            var pageCount = Math.ceil((-buffer - viewPos.y) / offset);
            var dy = pageCount * offset;
            if (items[i].y + dy < 0) {
              items[i].y = items[i].y + dy;
              var item = items[i].getComponent(this.componentName);
              var itemId = item.itemID - items.length * pageCount;
              item.updateItem(itemId);
            }
          }
        } else if (viewPos.y > buffer) {
          var _pageCount = Math.ceil((viewPos.y - buffer) / offset);
          var _dy = _pageCount * offset;
          if (items[i].y - _dy > -this.content.height) {
            items[i].y = items[i].y - _dy;
            var _item = items[i].getComponent(this.componentName);
            var _itemId = _item.itemID + items.length * _pageCount;
            _item.updateItem(_itemId);
          }
        }
      }
      this.lastContentPosY = this.scrollView.content.y;
    }
  });