// 游戏面板

let Panel = require("Panel")

var GuideUI = function(rootNode,resourceUrl) {
    Panel.call(this,rootNode,resourceUrl)
    this.binder = this.node.getComponent("GuideUIBinder")

    this.guideModel = null
    this.binder.ding.on(cc.Node.EventType.TOUCH_START, this.onTouch,this)
    this.binder.skipBtn.on(cc.Node.EventType.TOUCH_START, this.skipGuide,this)

    let pos = this.binder.ding.parent.convertToNodeSpaceAR(cc.v2(0,0))
    this.binder.ding.setPosition(pos)
}
GuideUI.prototype = Object.create(Panel.prototype); // 原型继承
GuideUI.prototype.constructor = GuideUI;            //修正
	
GuideUI.prototype.onShow = function(guideModel)
{
    this.guideModel = guideModel

    this.binder.ding.active = false
    this.binder.inputBlock.enable = true
    this.clickBtn = null
    this.buttonRect = null
    this.rectList = null

    let delayTime = this.guideModel.Delay
    if(delayTime > 0)
    {
        G.timer.scheduleOnce(this,this.display,delayTime)
    }else
    {
        this.display()
    }
}
GuideUI.prototype.display = function()
{
    this.binder.ding.active = true
    this.binder.ding._touchListener.swallowTouches = true
        
    this.binder.tipsLabel.string = this.guideModel.Desc
    let xyStr = this.guideModel.DescPosition.split(',')
    let x = parseInt(xyStr[0]), y = parseInt(xyStr[1])
    this.binder.tipsLabel.node.setPosition(x,y)

    this.binder.scheduleOnce( (()=>{
        let btnX = x + this.binder.tipsLabel.node.width
        this.binder.skipBtn.setPosition(btnX ,y - 50)
    }).bind(this),0)

    this.binder.bg.active = false
    if(this.guideModel.Button)
    {
        this.showButtonMask()
    }else if(this.guideModel.Position)
    {
        this.showMultiRectMask()
    }else{
        this.binder.bg.active = true
        this.binder.multiMask.node.active = false
        this.binder.buttonMaskNode.active = false
    }
}
GuideUI.prototype.showButtonMask = function()
{
    this.binder.multiMask.node.active = false
    let buttonMaskNode = this.binder.buttonMaskNode
    buttonMaskNode.active = true

    this.clickBtn = cc.find("Canvas/PanelRoot/" + this.guideModel.Button);
    if(!this.clickBtn)
    {
        console.log("导航找不到按钮" + this.guideModel.Button)
        return
    }
    
    let pos = this.clickBtn.convertToWorldSpace(cc.v2(0,0))
    let box = this.clickBtn.getBoundingBoxToWorld()
    let x = pos.x;
    let y = pos.y;

    this.buttonRect = new cc.Rect(pos.x,pos.y,box.width,box.height)
    //剪裁可点击区域
    let localPos = buttonMaskNode.parent.convertToNodeSpaceAR(pos)
    buttonMaskNode.setPosition(localPos);
    buttonMaskNode.setContentSize(box.size);
    //移动灰色背景
    let grayPos = this.binder.blackBg.parent.convertToNodeSpaceAR(cc.v2(0,0))
    this.binder.blackBg.setPosition(grayPos);
}
GuideUI.prototype.showMultiRectMask = function()
{
    let pointList = []
    let posStrArray = this.guideModel.Position.split('|')
    for(let j = 0; j < posStrArray.length; ++j)
    {
        let rowCol = posStrArray[j].split(',')
        let col = parseInt(rowCol[0]), row = parseInt(rowCol[1])
        pointList.push([col - 1,row - 1])
    }

    this.binder.buttonMaskNode.active = false
    this.binder.multiMask.node.active = true

    let stage = G.stageMgr.getCurStage()
    this.rectList = []
    let mask =  this.binder.multiMask
    let graphics = mask._graphics;
    graphics.clear(false);
    for(let i = 0; i < pointList.length; ++i)
    {
        let rowCol = pointList[i]
        let col = rowCol[0], row = rowCol[1]
        let pos = stage.getAnimalWorldPos(col,row)
        let x = pos.x
        let y = pos.y
        graphics.rect(x, y, 72, 72);

        this.rectList.push(cc.rect(x,y,72,72))
    }
    graphics.fill();
    if (mask._renderData) {
        mask.destroyRenderData(mask._renderData);
        mask._renderData = null;
    }
    mask._activateMaterial();
}

GuideUI.prototype.onHide = function()
{
}

GuideUI.prototype.onTouch = function(event)
{
    let pos = event.getLocation();
    if(this.buttonRect)
    {
        if (this.buttonRect.contains(pos) )
        {
            cc.log("in buttonRect")
            this.binder.ding._touchListener.swallowTouches = false
            this.binder.inputBlock.enable = false

            this.finishCurGuide()
        } else {
            cc.log("out of buttonRect")
            if(this.guideModel.CanSkip)
            {
                this.finishCurGuide()
            }
        }
    }else if(this.rectList)
    {
        for(let i = this.rectList.length - 1; i >= 0; --i)
        {
            let rect = this.rectList[i]
            if(rect.contains(pos))
            {
                cc.log("in rectList")
                this.binder.ding._touchListener.swallowTouches = false
                this.binder.inputBlock.enable = false

                this.finishCurGuide()
                return
            }
        }
        cc.log("out of rectList")
    }else{
        this.finishCurGuide()
    }
}

GuideUI.prototype.finishCurGuide = function()
{
    this.hide()
    G.eventCenter.dispatchEvent(g_eventTypes.guideEvent.finishCurStep)
}

GuideUI.prototype.skipGuide = function()
{
    this.hide()
    G.eventCenter.dispatchEvent(g_eventTypes.guideEvent.skipGuide)
}

module.exports = GuideUI