// 资源加载器

var ResourceLoader = function () {
    this.assetMap = {};
    this.needLoadList = [];
    this.loadedCount = 0;
    this.totalCount = 0;
    this.bLoading = false;
    this.curLoadTask = null;
    this.bError = false;
    this.loadPanelType = LoadPanelType.none;

    G.eventCenter.on(g_eventTypes.loadEvent.loadRes, this.loadRes, this);
    G.eventCenter.on(g_eventTypes.loadEvent.loadResource, this.onLoadRequest, this);
};
window.LoadPanelType = {
    none: 0,
    fullPanel: 1,
    windowPanel: 2
};

ResourceLoader.prototype = {
    constructor: ResourceLoader,

    // 获取资源
    getAsset(url) {
        return this.assetMap[url];
    },

    onLoadRequest (data) {
        data.urlList || (data.urlList = [ data.url ]);
        this.needLoadList.push(data);

        if (!this.bLoading) {
            this.bLoading = true;
            this.bError = false;

            G.timer.unschedule(this, this.startLoad);
            G.timer.scheduleOnce(this, this.startLoad, 0);

            data.loadPanelType && (this.loadPanelType = data.loadPanelType);

            switch (this.loadPanelType) {
                case LoadPanelType.none:
                    break;

                case LoadPanelType.fullPanel:
                    G.eventCenter.dispatchEvent(g_eventTypes.loadEvent.showFullPanel)
                    break;

                case LoadPanelType.windowPanel:
                        G.eventCenter.dispatchEvent(g_eventTypes.loadEvent.showLittlePanel)
                    break
            }
        }
    },

    startLoad () {
        this.totalCount = this.needLoadList.length;
        this.curLoadTask = this.needLoadList.shift();
        this.loadCurTask();
    },

    loadCurTask () {
        var self = this;
        cc.loader.loadResArray(this.curLoadTask.urlList,
            function(completedCount, totalCount, item) {
                self.progressCallback(completedCount, totalCount, item);
            },
            function(error, resource) {
                self.onCurTaskComplete(error, resource);
            }
        );
    },

    progressCallback (completedCount, totalCount, item) {
        var percent = (this.loadedCount + completedCount / totalCount) / this.totalCount;
        G.eventCenter.dispatchEvent(g_eventTypes.loadEvent.loadProgress, percent);
    },

    onCurTaskComplete (error, resource) {
        this.loadedCount++;

        if (error) {
            console.log("load assets error:", error);
            this.bError = true
        } else {
            if (this.curLoadTask.urlList.length == resource.length) {
                for (var index = 0; index < resource.length; index++) {
                    var path = this.curLoadTask.urlList[index];
                    var asset = resource[index];
                    asset ? this.assetMap[path] = asset : console.error("资源加载失败:" + path);
                }
            }
        }

        var callback = this.curLoadTask.callback;
        var target = this.curLoadTask.target;
        callback && callback.call(target, error, this.curLoadTask);

        if (0 == this.needLoadList.length) {
            this.needLoadList = [];
            this.loadedCount = 0;
            this.totalCount = 0;
            this.bLoading = false;
            this.curLoadTask = null;
            this.loadPanelType = LoadPanelType.none;

            G.eventCenter.dispatchEvent(g_eventTypes.loadEvent.loadProgress, 1);

            G.timer.scheduleOnce(this, function() {
                G.eventCenter.dispatchEvent(g_eventTypes.loadEvent.loadComplete, this.bError);
            }, 0);
        } else {
            this.curLoadTask = this.needLoadList.shift();
            this.loadCurTask();
            
            var progress = this.loadedCount / this.totalCount;
            G.eventCenter.dispatchEvent(g_eventTypes.loadEvent.loadProgress, progress);
        }
    },

    loadRes(url, type, cb) {
        var self = this;
        cc.loader.loadRes(url, type, function(err, asset) {
            err || (self.assetMap[url] = asset);
            cb && cb(err, asset);
        });
    },
};
module.exports = ResourceLoader;