// 普通关卡

let Stage = require("Stage")
let NodePool = require("NodePool")

var BlastStage = function(level) {
    Stage.call(this)

    this.curState = Stage.StageState.none
    this.bWin   = false // 胜利失败
    this.level  =  level || 43
    this.curScore   = 0
    this.model  = G.tableProvider.getData(g_resource.newLevelTable,this.level)

    this.dt = 0
    this.needDestroyList = []
    
    this.nodePool = new NodePool()// 缓存池 粒子特效
}
BlastStage.prototype = Object.create(Stage.prototype); // 原型继承
BlastStage.prototype.constructor = BlastStage;            //修正

// 预载
BlastStage.prototype.preload = function()
{
    this.curState = Stage.StageState.preloading

    let resList = [g_resource.NormalStagePrefab, g_resource.RoadPrefab]//, g_resource.GamePanel]

    let data = {}
    data.urlList = resList
    data.loadPanelType = LoadPanelType.fullPanel
    G.eventCenter.dispatchEvent(g_eventTypes.loadEvent.loadResource,data)

    G.eventCenter.once(g_eventTypes.loadEvent.loadComplete,this.init,this)
}
// 初始化
BlastStage.prototype.init = function()
{
    var scene = cc.director.getScene();
    let stageAsset = G.resourceLoader.getAsset(g_resource.NormalStagePrefab)
    let stageRootNode = cc.instantiate(stageAsset)
    stageRootNode.parent = scene
    this.display = stageRootNode.getComponent('NormalStageDisplay')

    this.car = stageRootNode.getChildByName('Car')
    
    this.display.node.on(cc.Node.EventType.TOUCH_START, this.onTouch,this)

    G.eventCenter.on(g_eventTypes.stageEvent.playEffect,this.playEffect,this)
    
    this.curState = Stage.StageState.gaming

    // G.soundMgr.playBackgroundMusic(g_resource.fightBgMusic)

    this.initRoad()

    // G.eventCenter.dispatchEvent(g_eventTypes.coreEventType.showUI,g_resource.GamePanel);
    
    G.eventCenter.dispatchEvent(g_eventTypes.stageEvent.stageStart)
    G.eventCenter.dispatchEvent(g_eventTypes.stageEvent.gameStart)
}
// 释放
BlastStage.prototype.destroy = function()
{
    this.display.node.destroy()
    this.display = null
    this.nodePool.destroy()
    // this.nodePool = null

    G.eventCenter.off(g_eventTypes.stageEvent.playEffect,this.playEffect,this)

    this.curState = Stage.StageState.released

    G.eventCenter.dispatchEvent(g_eventTypes.coreEventType.hideBannerAd)
}

// 触摸回调
BlastStage.prototype.onTouch = function(event)
{
    if(!this.curState == Stage.StageState.gaming)return

    var touch = event.touch;
    var target = this.display.mapRoot
    // 获取当前触摸点相对于按钮所在的坐标
    var locationInNode = target.convertToNodeSpaceAR(touch.getLocation());
    

}

BlastStage.prototype.initRoad = function()
{
    this.index = 0
    this.headRoad = null
    this.tailRoad = null

    this.buildRoad()
    this.buildRoad()
    this.buildRoad()
}
BlastStage.prototype.buildRoad = function()
{
    let info = {}
    info.index = ++this.index

    if (this.index == 1) {
        info.bottomAngle = 0
        info.angle = 90
        info.startPos = new cc.Vec3(0, 0, 0)
        info.preRoad = null

        this.headRoad = info
        this.tailRoad = info
        
        info.brPos = new cc.Vec3(g_config.roadWidth / 2, 0, 0)
        info.blPos = new cc.Vec3(g_config.roadWidth / -2, 0, 0)
    } else {
        info.preRoad = this.tailRoad
        this.tailRoad.nextRoad = info
        this.tailRoad = info

        info.bottomAngle = info.preRoad.topAngle
        info.angle = info.preRoad.nextAngle
        info.startPos = info.preRoad.endPos
        info.brPos = info.preRoad.trPos
        info.blPos = info.preRoad.tlPos
    }
    
    info.length = 5 // 随机道路长度
    info.nextAngle = 90 + 120 * (Math.random() - 0.5) // 随机下一段路的倾斜角度

    let radius = info.angle / 180 * Math.PI
    info.endPos = new cc.Vec3(0, 0, 0)
    info.endPos.x = info.startPos.x + Math.cos(radius) * info.length
    info.endPos.y = info.startPos.y + Math.sin(radius) * info.length

    info.topAngle = (info.angle - 180 + info.nextAngle) / 2
    let relativeAngle = (info.nextAngle - info.angle + 180) / 2
    let r = g_config.roadWidth / 2 / Math.sin(relativeAngle / 180 * Math.PI)
    let topAngleRadius = info.topAngle / 180 * Math.PI

    info.trPos = new cc.Vec3()
    info.trPos.x = info.endPos.x  + r * Math.cos(topAngleRadius)
    info.trPos.y = info.endPos.y  + r * Math.sin(topAngleRadius)
    
    info.tlPos = new cc.Vec3()
    info.tlPos.x = info.endPos.x  - r * Math.cos(topAngleRadius)
    info.tlPos.y = info.endPos.y  - r * Math.sin(topAngleRadius)

    info.mesh = this.createRoadMesh(info, cc.color(255, 100, 100))
    
    if (!info.node) {
        info.node = this.nodePool.get(g_resource.RoadPrefab)
        info.node.parent = this.display.node
    }
    info.node.setPosition(info.startPos.mul(100))
    let meshRenderer = info.node.getComponent(cc.MeshRenderer)
    meshRenderer.mesh = info.mesh
}

BlastStage.prototype.createRoadMesh = function(roadInfo, color)
{
    let corners = [
        roadInfo.blPos.sub(roadInfo.startPos),
        roadInfo.brPos.sub(roadInfo.startPos),
        roadInfo.trPos.sub(roadInfo.startPos),
        roadInfo.tlPos.sub(roadInfo.startPos),
    ];
    corners.push(corners[0].clone(), corners[1].clone(), corners[2].clone(), corners[3].clone())
    for(let i = 4; i < 8; ++i) {
        corners[i].z -= g_config.roadHeight
    }

    let leftRadius = (roadInfo.angle + 90) / 180 * Math.PI
    let rightRadius = leftRadius - Math.PI
    let faceNormals = [
        [  0,  0,  1 ], // TOP
        [ Math.cos(leftRadius),  Math.sin(leftRadius),  0 ],  // LEFT
        [  Math.cos(rightRadius),  Math.sin(rightRadius),  0 ], // RIGHT
    ];
    
    let positions = [
        corners[0].x, corners[0].y, corners[0].z,
        corners[1].x, corners[1].y, corners[1].z,
        corners[2].x, corners[2].y, corners[2].z,
        corners[3].x, corners[3].y, corners[3].z,

        corners[7].x, corners[7].y, corners[7].z,
        corners[4].x, corners[4].y, corners[4].z,
        corners[0].x, corners[0].y, corners[0].z,
        corners[3].x, corners[3].y, corners[3].z,

        corners[5].x, corners[5].y, corners[5].z,
        corners[6].x, corners[6].y, corners[6].z,
        corners[2].x, corners[2].y, corners[2].z,
        corners[1].x, corners[1].y, corners[1].z,
    ]

    let normals = [
        faceNormals[0][0], faceNormals[0][1], faceNormals[0][2], faceNormals[0][0], faceNormals[0][1], faceNormals[0][2], faceNormals[0][0], faceNormals[0][1], faceNormals[0][2], faceNormals[0][0], faceNormals[0][1], faceNormals[0][2],
        faceNormals[1][0], faceNormals[1][1], faceNormals[1][2], faceNormals[1][0], faceNormals[1][1], faceNormals[1][2], faceNormals[1][0], faceNormals[1][1], faceNormals[1][2], faceNormals[1][0], faceNormals[1][1], faceNormals[1][2], 
        faceNormals[2][0], faceNormals[2][1], faceNormals[2][2], faceNormals[2][0], faceNormals[2][1], faceNormals[2][2], faceNormals[2][0], faceNormals[2][1], faceNormals[2][2], faceNormals[2][0], faceNormals[2][1], faceNormals[2][2], 
    ]

    let indices = [
        0, 1, 3, 1, 2, 3, // top
        4, 5, 7, 5, 6, 7, // left
        8, 9, 11, 9, 10, 11, // right
        // 1, 7, 5, 1, 3, 7, // bottom,
        // 0, 1, 2, 1, 3, 2, // front
        // 4, 5, 6, 5, 7, 6, // back
    ]

    let minX = 9999
    let maxX = -9999
    let minY = 9999
    let maxY = -9999
    let minZ = -g_config.roadHeight
    let maxZ = 0
    for(let j = 0; j < corners.length; ++j) {
        let pos = corners[j]
        if (pos.x < minX)  minX = pos.x
        if (pos.y < minY)  minY = pos.y
        if (pos.x > maxX) maxX = pos.x
        if (pos.y > maxY) maxY = pos.y
    }
    
    let minPos = cc.v3(minX, minY, minZ);
    let maxPos = cc.v3(maxX, maxY, maxZ);
    let temp = cc.v3(maxPos.x - minPos.x, maxPos.y - minPos.y, maxPos.z - minPos.z)
    let boundingRadius = Math.sqrt(temp.x * temp.x + temp.y * temp.y + temp.z * temp.z);
    
    let gfx = cc.gfx;
    let vfmt = new gfx.VertexFormat([
        { name: gfx.ATTR_POSITION, type: gfx.ATTR_TYPE_FLOAT32, num: 3 },
        { name: gfx.ATTR_NORMAL, type: gfx.ATTR_TYPE_FLOAT32, num: 3 },
        { name: gfx.ATTR_COLOR, type: gfx.ATTR_TYPE_UINT8, num: 4, normalize: true },
    ]);

    let colors = [];
    for (let i = 0; i < positions.length; i++) {
        colors.push(color);
    }

    let mesh = new cc.Mesh();
    mesh.init(vfmt, positions.length);
    mesh.setVertices(gfx.ATTR_POSITION, positions);
    mesh.setVertices(gfx.ATTR_NORMAL, normals);
    mesh.setVertices(gfx.ATTR_COLOR, colors);
    mesh.setIndices(indices);
    mesh.setBoundingBox(minPos, maxPos);

    return mesh;
}

// 关卡结束
BlastStage.prototype.gameOver = function()
{
    if(this.curState == Stage.StageState.over){
        console.log('重复over了')
        return
    }
    this.curState = Stage.StageState.over
}

BlastStage.prototype.checkOver = function()
{
    let bPass = true
    for(let k in this.collectionList)
    {
        let data = this.collectionList[k]
        if(data.collectNum < data.targetNum)
        {
            bPass = false
            break
        }
    }

    if(bPass)// 通关
    {

    }else
    {
        if(this.step >= this.stepLimit)// 失败
        {

        }
    }
}

// 播放特效
BlastStage.prototype.playEffect = function(cell, effect, delayTime, playTime,rotation, scaleX)
{
    if(!playTime)playTime = 2.2
    let realFunc = ()=>{
        if(this.curState == Stage.StageState.released)return
        let node = this.nodePool.get(effect)
        node.parent = this.display.effectLayer
        let x = (cell.col + 0.5) * g_config.GRID_WIDTH
        let y = (cell.row + 0.5) * g_config.GRID_HEIGHT
        node.setPosition(x,y)
        if(rotation)node.rotation = rotation
        if(scaleX)node.scaleX = scaleX

        let particleList = node.getComponentsInChildren(cc.ParticleSystem)
        for(let k in particleList)
        {
            particleList[k].resetSystem()
        }
        let animationList = node.getComponentsInChildren(cc.Animation)
        for(let k in animationList)
        {
            animationList[k].play()
        }

        G.timer.scheduleOnce(this,()=>{
            this.nodePool.put(node)
        }, playTime)
    }

    if(delayTime && delayTime > 0)
    {
        G.timer.scheduleOnce(this,realFunc, delayTime) 
    }else
    {
        realFunc.call(this)
    }
},
BlastStage.prototype.getLevel = function()
{
    return this.level
}
BlastStage.prototype.getCurScore = function()
{
    return this.curScore
}
BlastStage.prototype.update = function(dt)
{
    if (this.curState != Stage.StageState.gaming) return

    this.dt += dt

    let rate = this.dt / 3

    let x = this.headRoad.startPos.x + (this.headRoad.endPos.x - this.headRoad.startPos.x) * rate
    let y = this.headRoad.startPos.y + (this.headRoad.endPos.y - this.headRoad.startPos.y) * rate
    this.car.x = 100 * x
    this.car.y = 100 * y

    if (rate >= 1) {
        this.dt = 0

        this.buildRoad()

        this.needDestroyList.push(this.headRoad)

        this.headRoad = this.headRoad.nextRoad
        this.headRoad.preRoad = null

    }

    if (this.needDestroyList.length >=2) {
        let road = this.needDestroyList.shift()
        road.nextRoad = null
        road.node.destroyAllChildren()
        road.node.destroy()
    }
}

module.exports = BlastStage;