// 登录

var Login = function () {
    this.code = null
    this.btnList = null
    this.userInfoBtnList = []

    G.eventCenter.on(g_eventTypes.loginEvent.loginRequest, this.login, this);
    G.eventCenter.on(g_eventTypes.loginEvent.checkAuthorize, this.checkAuthorize, this);
    G.eventCenter.on(g_eventTypes.loginEvent.destroyUserInfoButton, this.destroyUserInfoButton, this);
}

Login.prototype = {
    constructor: Login,

    // 微信登录
    login()
    {
        if(CC_WECHATGAME)
        {
            let self = this
            wx.getSetting({
                success (settingRes) {
                    console.log(settingRes.authSetting);
                    if (settingRes.authSetting["scope.userInfo"]) {
                        console.log("用户已授权");
                        wx.getUserInfo({
                            success(res){
                                console.log(res);
                                G.getMyUserInfo().setWxUserInfo(res.userInfo)
                                self.wxLogin(res.iv,res.encryptedData,res.signature)
                            }
                        });
                    }else {
                        console.log("用户未授权");
                        self.wxLogin()
                    }
                }
            })
            
        }
    },
    wxLogin(iv, encryptedData, signature, successCb)
    {
        let self = this
        wx.login({
            success(res) {
                if (res.code) {
                    self.serverLogin(res.code,iv,encryptedData,signature, successCb)
                }
            },
            fail(){
                G.eventCenter.dispatchEvent(g_eventTypes.UIEvent.showTips,g_string.wxLoginFailed);
            }
        })
    },
    
    // 服务器登录
    serverLogin(code,iv,encryptedData,signature, successCb)
    {
        let self = this
        let shareType = 0
        let scenceID = G.getSceneID()
        if(scenceID == 1007 || scenceID == 1008)shareType = scenceID
        this.code = code
        let fromUid = G.getInviterUid()
        
        G.eventCenter.dispatchEvent(g_eventTypes.loginEvent.startLoginServer);
        let data = {
            code: code,
            platform : (cc.sys.os == cc.sys.OS_ANDROID)? 0 : 1,
            shareType : shareType,
        }
        if(iv)
        {
            data.encryptedData = encryptedData
            data.iv = iv
            data.signature = signature
        }
        if(fromUid)
        {
            data.fromUid = fromUid
        }
        wx.request({
            url: 'https://dev.988wan.com/api/sp/player/login',
            method: 'POST',
            data: data,
            success(res)
            {
                let msg = res.data
                if(msg)
                {
                    if(msg.status == 0)
                    {
                        self.onServerLoginSuccess(msg.data, successCb)
                    }else
                    {
                        G.eventCenter.dispatchEvent(g_eventTypes.UIEvent.showTips,g_string.loginFailed);
                    }
                }
            },
            fail(res)
            {
                console.log('serverLogin fail' + res)
            }
        })
    },
    onServerLoginSuccess(data, successCb)
    {
        let openId = data.openId
        let uid = data.uid
        let score = data.score
        let inviteFriendNum = parseInt(data.inviteCount)

        G.getMyUserInfo().setOpenid(openId)
        G.getMyUserInfo().uid = uid
        G.getMyUserInfo().setInviteFriendNum(inviteFriendNum)
        if(score < 1200000)
        {
            G.getMyUserInfo().bestScore = score
            localStorage.setItem("bestScore", score);
        }

        G.eventCenter.dispatchEvent(g_eventTypes.loginEvent.loginSuccess);

        if(successCb)successCb()
        console.log('serverLogin success openid uid', openId, uid)
    },

    
    // 获取微信用户信息
    checkAuthorize(btnList,unforcedBtnList)
    {
        if(CC_WECHATGAME)
        {
            let self = this

            wx.getSetting({
                success (res) {
                    console.log(res.authSetting);
                    if (res.authSetting["scope.userInfo"]) {
                        
                    }else {
                        console.log("用户未授权");
                        if(btnList)
                        {
                            self.createUserInfoBtnList(btnList,unforcedBtnList)
                        }else
                        {
                            self.createFullScreenUserInfoBtn()
                        }
                    }
                }
            })
        }
    },
    // 创建获取用户信息按钮
    createFullScreenUserInfoBtn()
    {
        let self = this
        
        let sysInfo = wx.getSystemInfoSync();
        //获取微信界面大小
        let width = sysInfo.screenWidth;
        let height = sysInfo.screenHeight;
        let button = wx.createUserInfoButton({
            type: 'text',
            text: '',
            style: {
                left: 0,
                top: 0,
                width: width,
                height: height,
                backgroundColor: '#00000000',//最后两位为透明度
                color: '#ffffff',
                fontSize: 20,
                textAlign: "center",
                lineHeight: height,
            }
        });
        button.onTap((res) => {
            if (res.userInfo) {
                console.log("用户授权:", res);
                button.destroy();
                
                G.getMyUserInfo().setWxUserInfo(res.userInfo)
                self.serverLogin(self.code,res.iv,res.encryptedData,res.signature)
            }else {
                console.log("用户拒绝授权:", res);
            }
        });
    },
    // 创建多个按钮
    createUserInfoBtnList(btnList,unforcedBtnList)
    {

        if(!wx.createUserInfoButton)
        {
            return wx.showModal({
                title: '提示',
                content: '当前微信版本过低，请升级到最新微信版本后重试。'
            })
        }
        if(btnList)
        {
            for(let k in btnList)
            {
                let btn = btnList[k]
                this.createUserInfoBtn(btn,false)
            }
        }
        if(unforcedBtnList)
        {
            for(let k in unforcedBtnList)
            {
                let btn = unforcedBtnList[k]
                this.createUserInfoBtn(btn,true)
            } 
        }
    },
    createUserInfoBtn(btn,bUnfored)
    {
        let self = this

        let sysInfo = wx.getSystemInfoSync();
        let rect = btn.getBoundingBoxToWorld()
        let ratio = cc.view._devicePixelRatio
        let scale = cc.view.getScaleX()
        let factor = scale / ratio
        let point = cc.v2(rect.x,rect.y)
        cc.view._convertPointWithScale(point)
        let button = wx.createUserInfoButton({
            type: 'text',
            text: '',
            style: {
                left: rect.x * factor,
                top: sysInfo.screenHeight - (rect.y + rect.height) * factor,
                width: rect.width * factor,
                height: rect.height * factor,
                // backgroundColor: '#ff00ffff',
                backgroundColor: '#00000000',//最后两位为透明度
                color: '#ffffff',
                fontSize: 20,
                textAlign: "center",
                lineHeight: rect.height,
            }
        });
        button.onTap((res) => {
            if (res.userInfo) {
                console.log("用户授权:", res);

                G.getMyUserInfo().setWxUserInfo(res.userInfo)
                G.eventCenter.dispatchEvent(g_eventTypes.loginEvent.authorized)

                self.uploadUserInfo(res.iv, res.encryptedData, res.signature, ()=>{
                    btn.emit('click')
                })

                for(let k in self.userInfoBtnList)
                {
                    self.userInfoBtnList[k].destroy()
                }
                self.userInfoBtnList = []
            }else {
                console.log("用户拒绝授权:", res);
                if(bUnfored)
                {
                    let index = self.userInfoBtnList.indexOf(button)
                    self.userInfoBtnList.splice(index, 1)
                    button.destroy()

                    btn.emit('click')
                    G.getMyUserInfo().addRefuseAuthorizeTimes()
                    G.eventCenter.dispatchEvent(g_eventTypes.loginEvent.authorizeDenied)
                }
            }
        });
        this.userInfoBtnList.push(button)
    },
    uploadUserInfo(iv,encryptedData,signature, successCb)
    {
        let self = this

        let upload = function(code)
        {
            let shareType = 0
            let scenceID = G.getSceneID()
            if(scenceID == 1007 || scenceID == 1008)shareType = scenceID
            
            let data = {
                code: code,
                platform : (cc.sys.os == cc.sys.OS_ANDROID)? 0 : 1,
                shareType : shareType,
                iv : iv,
                encryptedData : encryptedData,
                signature : signature,
            }
            wx.request({
                url: 'https://dev.988wan.com/api/sp/player/login',
                method: 'POST',
                data: data,
                success(res)
                {
                    let msg = res.data
                    if(msg)
                    {
                        if(msg.status == 0)
                        {
                            if(successCb)successCb()
                        }else
                        {
                            G.eventCenter.dispatchEvent(g_eventTypes.UIEvent.showTips, 'uploadUserInfo failed');
                        }
                    }
                },
                fail(res)
                {
                    console.log('uploadUserInfo fail' + res)
                }
            })
        }

        // wx.checkSession({
        //     success() {
        //       // session_key 未过期，并且在本生命周期一直有效
        //       upload.call(self,self.code)
        //     },
        //     fail() {
        //       // session_key 已经失效，需要重新执行登录流程
        //         wx.login({
        //             success(res) {
        //                 if (res.code) {
        //                     upload.call(self,res.code)
        //                 }
        //             },
        //             fail(){
        //                 G.eventCenter.dispatchEvent(g_eventTypes.UIEvent.showTips,g_string.wxLoginFailed);
        //             }
        //         })
        //     }
        // })

        wx.login({
            success(res) {
                if (res.code) {
                    upload.call(self,res.code)
                }
            },
            fail(){
                G.eventCenter.dispatchEvent(g_eventTypes.UIEvent.showTips,g_string.wxLoginFailed);
            }
        })
    },
    destroyUserInfoButton()
    {
        for(let k in this.userInfoBtnList)
        {
            this.userInfoBtnList[k].destroy()
        }
        this.userInfoBtnList = []
    },
}

module.exports = Login;